import 'package:flutter/material.dart';
import 'package:awesome_notifications/awesome_notifications.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:http/http.dart' as http;
import 'package:firebase_core/firebase_core.dart';
import 'package:firebase_messaging/firebase_messaging.dart';
import 'dart:convert';
import 'dart:io';

String host = 'https://app.nativenotify.com';

class NativeNotify {
  static void initialize(
      flutterAppId, flutterAppToken, firebaseServerKey) async {
    final prefs = await SharedPreferences.getInstance();

    await prefs.setInt('flutterAppId', flutterAppId);
    await prefs.setString('flutterAppToken', flutterAppToken);

    final flutterDeviceId = prefs.getInt('flutterDeviceId');
    final firebaseServerKeyShared = prefs.getString('firebaseServerKey');

    if (firebaseServerKey == null) {
      if (flutterDeviceId == null) {
        try {
          var response = await http.post(Uri.parse('$host/api/flutter/device'),
              headers: {
                HttpHeaders.contentTypeHeader: 'application/json',
              },
              body: jsonEncode({
                'flutterAppId': '$flutterAppId',
                'flutterAppToken': flutterAppToken
              }));

          var jsonData = jsonDecode(response.body);

          await jsonData.forEach((key, value) async => {
                if (key == 'flutter_device_id')
                  {await prefs.setInt('flutterDeviceId', value)}
              });

          print(
              "You can send Native Notify push notifications now. You're registered!");

          getPushNotification();
        } catch (e) {
          // print(e);
        }
      } else {
        getPushNotification();
      }
    } else {
      String? firebaseToken;

      try {
        await Firebase.initializeApp();
        firebaseToken = await FirebaseMessaging.instance.getToken();
      } catch (e) {
        firebaseToken = null;
      }

      if (flutterDeviceId == null ||
          firebaseServerKeyShared == 'your-firebase-server-key' ||
          firebaseServerKeyShared == null ||
          firebaseServerKeyShared == '') {
        if (firebaseServerKey != null) {
          await prefs.setString('firebaseServerKey', firebaseServerKey);
        }

        try {
          var response = await http.post(Uri.parse('$host/api/flutter/device'),
              headers: {
                HttpHeaders.contentTypeHeader: 'application/json',
              },
              body: jsonEncode({
                'flutterAppId': '$flutterAppId',
                'flutterAppToken': flutterAppToken,
                'firebaseToken': firebaseToken,
                'firebaseServerKey': firebaseServerKey
              }));

          var jsonData = jsonDecode(response.body);

          await jsonData.forEach((key, value) async => {
                if (key == 'flutter_device_id')
                  {await prefs.setInt('flutterDeviceId', value)}
              });

          print(
              "You can send Native Notify push notifications now. You're registered!");
        } catch (e) {
          // print(e);
        }
      } else {
        getPushNotification();
      }
    }

    // need to figure out how to change null to an actual push icon that will work for android
    AwesomeNotifications().initialize(null, [
      NotificationChannel(
          channelKey: 'mass',
          channelName: 'Native Notify Mass Notifications',
          channelDescription: "Notifications for all users",
          defaultColor: const Color(0XFF9050DD),
          ledColor: Colors.white,
          playSound: true,
          enableLights: true,
          enableVibration: true,
          importance: NotificationImportance.High)
    ]);

    AwesomeNotifications().isNotificationAllowed().then((isAllowed) {
      if (!isAllowed) {
        AwesomeNotifications().requestPermissionToSendNotifications();
      }
    });

    FirebaseMessaging.onMessage.listen(_firebaseMessagingHandler);
    FirebaseMessaging.onBackgroundMessage(_firebaseMessagingHandler);
  }
}

Future<void> _firebaseMessagingHandler(RemoteMessage message) async {
  await Firebase.initializeApp();

  var deviceInfo = await jsonDecode(message.data['deviceInfo']);

  final flutterDeviceId = deviceInfo['flutterDeviceId'];
  final flutterAppId = deviceInfo['flutterAppId'];
  final flutterAppToken = deviceInfo['flutterAppToken'];

  try {
    await http.read(Uri.parse(
        '$host/api/flutter/notification/$flutterDeviceId/$flutterAppId/$flutterAppToken'));
  } catch (e) {
    // print(e);
  }

  AwesomeNotifications().createNotificationFromJsonData(message.data);
}

void checkForNotifications() async {
  final prefs = await SharedPreferences.getInstance();

  final flutterDeviceId = prefs.getInt('flutterDeviceId');
  final flutterAppId = prefs.getInt('flutterAppId');
  final flutterAppToken = prefs.getString('flutterAppToken');

  try {
    var response = await http.read(Uri.parse(
        '$host/api/flutter/notification/$flutterDeviceId/$flutterAppId/$flutterAppToken'));

    List<dynamic> notifications = jsonDecode(response);

    for (var i = 0; i < notifications.length; i++) {
      if (notifications[i]['notification_big_picture_url'] == null ||
          notifications[i]['notification_big_picture_url'] == '') {
        AwesomeNotifications().createNotification(
          content: NotificationContent(
              id: notifications[i]['flutter_notification_id'],
              channelKey: 'mass',
              title: notifications[i]['notification_title'],
              body: notifications[i]['notification_body']),
        );
      } else {
        AwesomeNotifications().createNotification(
          content: NotificationContent(
              id: notifications[i]['flutter_notification_id'],
              channelKey: 'mass',
              title: notifications[i]['notification_title'],
              body: notifications[i]['notification_body'],
              bigPicture: notifications[i]['notification_big_picture_url'],
              notificationLayout: NotificationLayout.BigPicture),
        );
      }
    }
  } catch (e) {
    // print(e);
  }
}

void getPushNotification() {
  Future.delayed(const Duration(seconds: 30), () {
    return checkForNotifications();
  }).whenComplete(() => getPushNotification());
}
